/* eslint-disable no-console */
/* eslint-disable */
import {
  IDictionary, Metadata, Http,
  Loader,
} from '@zeedhi/core';
import {
  Button, Form, Select, TextInput,
  DialogService, AlertService, Textarea,
  Text,
} from '@zeedhi/common';
import { getRouteParams, getRoutePath } from '../router';
import { IterableComponentRender, ICardTitle } from '@zeedhi/teknisa-components-common';

export class TarefasSolicitacoesController {
  public modalSolicitacoes = Loader.getInstance('ModalTarefasSolicitacoesController');
  
  /* --------------- Tela: /solicitacoesDeTarefas --------------- */
  
  // Arrays de informações referentes a cada iterable de solicitação
  public requestsData: IDictionary = {
    newRequests: [],
    requestsInProgress: [],
    requestsMade: [],
    completedRequests: [],
  };
  
  // Protótipo do data de solicitações que pode ser usado para qualquer Iterable da aba "Solicitações"
  public requestData(nrsolicitacao: number, nmsolicitacao: string, nmlocalcliente: string, nmlocalclientepai: string, dtsolicitacao: string, nmfuncionario: string){
    return {
      value: nrsolicitacao,
      name: nmsolicitacao,
      location: nmlocalcliente,
      parentLocation: nmlocalclientepai,
      time: dtsolicitacao,
      taskResponsible: nmfuncionario,
    };
  }

  // Busca todas as solicitações e monta os iterables específicos para cada situação da solicitação
  public getRequests() {
    const newRequests = Metadata.getInstance<IterableComponentRender>('newRequestsIterable');
    const requestsInProgress = Metadata.getInstance<IterableComponentRender>('requestsIPIterable');
    const requestsMade = Metadata.getInstance<IterableComponentRender>('requestsMadeIterable');
    const completedRequests = Metadata.getInstance<IterableComponentRender>('CompletedRequestsIterable');

    Http.get('/getRequests').then((response: IDictionary) => {
      response.data.dataset.info.forEach((database: IDictionary) => {
        if (database.IDSITUACAO === '1') {
          this.requestsData.newRequests.push(this.requestData(database.NRSOLICITACAO, database.NMSOLICITACAO,database.NMLOCALCLIENTE,  database.NMLOCALCLIENTEPAI, database.DTSOLICITACAO, database.NMFUNCIONARIO));
        } else  if (database.IDSITUACAO === '2') {
          this.requestsData.requestsInProgress.push(this.requestData(database.NRSOLICITACAO, database.NMSOLICITACAO,database.NMLOCALCLIENTE,  database.NMLOCALCLIENTEPAI, database.DTSOLICITACAO, database.NMFUNCIONARIO));
        } else  if (database.IDSITUACAO === '3') {
          this.requestsData.requestsMade.push(this.requestData(database.NRSOLICITACAO, database.NMSOLICITACAO,database.NMLOCALCLIENTE,  database.NMLOCALCLIENTEPAI, database.DTSOLICITACAO, database.NMFUNCIONARIO));
        } else  if (database.IDSITUACAO === '4') {
          this.requestsData.completedRequests.push(this.requestData(database.NRSOLICITACAO, database.NMSOLICITACAO,database.NMLOCALCLIENTE,  database.NMLOCALCLIENTEPAI, database.DTSOLICITACAO, database.NMFUNCIONARIO));
        }
      });
      newRequests.datasource.updateData(this.requestsData.newRequests);
      requestsInProgress.datasource.updateData(this.requestsData.requestsInProgress);
      requestsMade.datasource.updateData(this.requestsData.requestsMade);
      completedRequests.datasource.updateData(this.requestsData.completedRequests);
    });
  }
  
  // Busca os tipos de solicitação e monta o iterable
  public getRequestTypes() {
 
    Http.get('/getRequestTypes').then((response: IDictionary) => {
      response.data.dataset.info.forEach((database: IDictionary) => {
         Metadata.getInstance<IterableComponentRender>('requestTypesIterable').datasource.data.push({
          value: database.NRTIPOSOLICITACAO,
          name: database.NMTIPOSOLICITACAO,
          priority: database.IDPRIORIDADEPADR,
        });
      });
    });
  }
  
  // Inserir Tipo de Solicitação
  public insertRequestType() {
    if (Metadata.getInstance<Form>('typeRequestForm').validate()) {
      const params = {
        NMTIPOSOLICITACAO: Metadata.getInstance<TextInput>('typeRequestName').value,
        IDPRIORIDADEPADR: Metadata.getInstance<Select>('typeRequestPriority').value,
      };
      Http.post('/insertRequestType', { params }).then((response: IDictionary) => {
        if(response.data.dataset.info.message != 'Inserido com sucesso') {
          DialogService.show({
            name: 'insertFailed',
            type: 'error',
            title: 'Erro!',
            text: 'Não foi possível inserir um novo tipo de solicitação.',
          });
        } else {
          AlertService.show ({
            name: 'insertConcluded',
            text: 'Tipo de solicitação inserido com sucesso.',
            color: 'success',
            top: true,
            left: false,
          });
        }
      });
      this.modalSolicitacoes.closeInsertTypeModal();
      Metadata.getInstance<IterableComponentRender>('requestTypesIterable').reload();

      setTimeout(() => {
       this.getRequestTypes();
      }, 1000);
    }
  }
  
  // Seta os valores no modal editar tipo de solicitação
  public setRequestType({ component }: { component:Button }) {
    Promise.all([this.modalSolicitacoes.openInsertTypeModal()]).then(() => {
      const typeParams = component.name.split('_');
      const priority = typeParams[3];
      const prioritySelect = Metadata.getInstance<Select>('typeRequestPriority');
      const typeId = typeParams[1];
      
      if(priority === 'Baixa'){
        prioritySelect.setValue('1');
      } else if(priority === 'Média') {
        prioritySelect.setValue('2');
      } else if(priority ==='Alta') {
        prioritySelect.setValue('3');
      } else {
        prioritySelect.setValue('4');
      }
      Metadata.getInstance<TextInput>('typeRequestName').value = typeParams[2];
      Metadata.getInstance<Text>('typeRequestTitle').text = 'Editar Tipo de Solicitação';
      
       Metadata.getInstance<Button>('saveTypeButton').events = {
         click : () => this.updateRequestType(typeId),
       };
    });
  }
  
  // Edita os valores do tipo de solicitação
  public updateRequestType(typeId: string) {
    const params = {
      NRTIPOSOLICITACAO: typeId,
      NMTIPOSOLICITACAO: Metadata.getInstance<TextInput>('typeRequestName').value,
      IDPRIORIDADEPADR: Metadata.getInstance<Select>('typeRequestPriority').value
    };
    Http.post('/updateRequestType', { params }).then((response: IDictionary) => {
        if(response.data.dataset.info.message != 'Editado com sucesso') {
          DialogService.show({
            name: 'updateFailed',
            type: 'error',
            title: 'Erro!',
            text: 'Não foi possível editar o tipo de solicitação.',
          });
        } else {
          AlertService.show ({
            name: 'updateConcluded',
            text: 'Tipo de solicitação editada com sucesso.',
            color: 'success',
            top: true,
            left: false,
          });
        }
      });
      this.modalSolicitacoes.closeInsertTypeModal();
      Metadata.getInstance<IterableComponentRender>('requestTypesIterable').reload();

      setTimeout(() => {
       this.getRequestTypes();
      }, 1000);
  }
  

  // Muda o label do botão "Ver descrição" na aba "Histórico"
  public changeDescriptionButton({ component }: { component:Button }) {
    const collapaseCard = component.parent as IDictionary;

    (component.label === 'Ver Descrição') ? (
      component.label = 'Fechar Descrição',
      collapaseCard.expanded = true
      ) : (
        component.label = 'Ver Descrição',
        collapaseCard.expanded = false
      );
  }
  
  /* --------------- Tela: /classificarSolicitacao --------------- */
  
  /*  Algumas funções que deixei de usar porque na tela troquei o component "Form" por conjunto de "Texts"
  
  // Inicializa os dados para colocar nos selects
  public initializeData() {
    this.getLocations();
    this.getEmployees();
    this.getRequestTypeSelect();
  }
  
  // Busca e salva no array os locais
  public getLocations() {
    const selectObject = Metadata.getInstance<Select>('requestLocation');
    Http.get('/getLocals').then((response: IDictionary) => {
      response.data.dataset.info.forEach((database: IDictionary) => {
        selectObject.datasource.data.push({
          value: database.NRLOCALCLIENTE,
          name: database.NMLOCALCLIENTE
        });
      })
    })
  }
  
  // Busca e salva no array os funcionários
  public getEmployees() {
    const openedBySelect = Metadata.getInstance<Select>('requestOpenedBy');
    const taskResponsibleSelect = Metadata.getInstance<Select>('taskResponsible');
    Http.get('/usersList').then((response: IDictionary) => {
      response.data.dataset.info.forEach((database: IDictionary) => {
      openedBySelect.datasource.data.push({
          value: database.NRFUNCIONARIO,
          name: database.NMFUNCIONARIO
        });
        
        taskResponsibleSelect.datasource.data.push({
          value: database.NRFUNCIONARIO,
          name: database.NMFUNCIONARIO
        });
      })
    })
  }
  
  // Busca os tipos de requisição, nesse caso para montar o select
  public getRequestTypeSelect() {
    const selectObject = Metadata.getInstance<Select>('requestType');
    Http.get('/getRequestTypes').then((response: IDictionary) => {
      response.data.dataset.info.forEach((database: IDictionary) => {
        selectObject.datasource.data.push({
          value: database.NRTIPOSOLICITACAO,
          name: database.NMTIPOSOLICITACAO,
          priority: database.IDPRIORIDADEPADR,
        });
      });
    });
  }
  */
  
  // array de selects
  public requestSelectsData: IDictionary = {
    requestLocation: [],
    requestType: [],
    requestOpenedBy: [],
    taskResponsible: [],
    checkList: []
  }
  
  // Seta em cada select os valores do array requestSelectsData
  public setSelectsData({ component }: { component:Select }) {
    component.datasource.data = this.requestSelectsData[component.name];
  }
  
  // Título da página ( Usando assim porque será usada tb no modal)
  public requestTitle: string = '';
  
  public requestDescription: string = '';
 
  // Parametros usados para a classificação da solicitação (São usados assim pq tenho o modal sendo aberto por dois locais diferentes)
  public requestParams: IDictionary =  { 
    requestId: '',
    requestNrOpenedBy: '',
    requestOpenOn: '',
    requestNrLocation: '',
  }

  // Busca a informação da solicitação específica e mostra nos campos
  public getRequestInfo() {
    const routeParams = getRouteParams();
    const { id } = routeParams;

    Http.get(`/getRequestInfo?nrsolicitacao=${id}`).then((response: IDictionary) => {
      const database = response.data.dataset.info[0];
      const classifyButton = Metadata.getInstance<Button>('classifyButton');
      this.requestTitle = database.NMSOLICITACAO;
      this.requestDescription = database.DSSOLICITACAO;
  
      this.requestParams.requestId = database.NRSOLICITACAO;
      Metadata.getInstance<ICardTitle>('requestTitle').title = database.NMSOLICITACAO;
      Metadata.getInstance<Text>('requestOpenOn').text = database.DTSOLICITACAO;
      this.requestParams.requestOpenOn = database.DTSOLICITACAO;
      Metadata.getInstance<Text>('requestStatus').text = database.IDSITUACAO;
      Metadata.getInstance<Text>('requestLocation').text = database.NMLOCALCLIENTE;
      Metadata.getInstance<Text>('requestNrLocation').text = database.NRLOCALCLIENTE;
      this.requestParams.requestNrLocation = database.NRLOCALCLIENTE;
      Metadata.getInstance<Text>('requestType').text = database.NMTIPOSOLICITACAO;
      Metadata.getInstance<Text>('requestPriority').text = database.IDPRIORIDADE;
      Metadata.getInstance<Text>('requestOpenedBy').text = database.NMFUNCABERTURA;
      Metadata.getInstance<Text>('requestNrOpenedBy').text = database.NRFUNCABERTURA;
      this.requestParams.requestNrOpenedBy = database.NRFUNCABERTURA;
      Metadata.getInstance<Text>('taskResponsible').text = database.NMFUNCRESPONSAVEL;
      Metadata.getInstance<Text>('requestDescription').text = database.DSSOLICITACAO;
      
      (database.IDSITUACAO != 'Pendente') ? (
          classifyButton.label = 'Classificado',
          classifyButton.disabled = true
        ) : null 
    });
  }
  
  public getRequestHistoric() {
    const routeParams = getRouteParams();
    const { id } = routeParams;
    
    const requestHistoric: IDictionary[] = [];
    
    Http.get(`/getRequestHistoric?nrsolicitacao=${id}`).then((response: IDictionary) => {
      response.data.dataset.info.forEach((database: IDictionary) => {
        requestHistoric.push({
          value: database.NRSOLICITACAOHISTORICO,
          name: database.NMSOLICITACAO,
          status: database.IDTIPOHISTORICO,
          requestedBy: database.NMFUNCIONARIO,
          date: database.DTHISTSOLICITACAO,
          location: database.NMLOCALCLIENTE,
          description: database.DSSOLICITACAO,
        })
      });
      Metadata.getInstance<IterableComponentRender>('requestTypesIterable').datasource.updateData(requestHistoric);
    });
  }
  
  /* ---------------modal : Classificar Solicitação --------------- */
 
 
  // Abre o modal e seta os valores determinados
  public openClassifyModal({ component } : { component:Button }) {
    Promise.all([this.modalSolicitacoes.openClassifyModal()]).then(() => {
      if( component.label != 'Classificar') {
        Metadata.getInstance<TextInput>('classifyTitle').value = this.requestTitle;
        Metadata.getInstance<Textarea>('classifyDescription').value = this.requestDescription;
      } else {
        this.fillModal(component.name);
      }
    });
  }
 
 // Quando aberto pela tela te solicitações ele faz a requisição e seta os valores
  public fillModal(id: string) {
     Http.get(`/getRequestInfo?nrsolicitacao=${id}`).then((response: IDictionary) => {
        const database = response.data.dataset.info[0];
        
        this.requestParams.requestId = database.NRSOLICITACAO;
        this.requestParams.requestNrOpenedBy = database.NRFUNCABERTURA;
        this.requestParams.requestOpenOn = database.DTSOLICITACAO;
        this.requestParams.requestNrLocation = database.NRLOCALCLIENTE;
        
        Metadata.getInstance<TextInput>('classifyTitle').value = database.NMSOLICITACAO;
        Metadata.getInstance<Textarea>('classifyDescription').value = database.DSSOLICITACAO;
     });
  }
  
  // Muda o campo prioridade
  public setPriority({ component }: { component:Select }) {
    const priority = component.selectValue.priority;
    const prioritySelect = Metadata.getInstance<Select>('classifyPriority');
    
    if(priority === 'Baixa'){
      prioritySelect.setValue('1');
    } else if(priority === 'Média') {
      prioritySelect.setValue('2');
    } else if(priority ==='Alta') {
      prioritySelect.setValue('3');
    } else {
      prioritySelect.setValue('4');
    }
  }
  
  // Classificar solicitação
  public classifyRequest() {
    if(Metadata.getInstance<Form>('classifyForm').validate()) {
      const params = {
        NRSOLICITACAO: this.requestParams.requestId,
        NMSOLICITACAO: Metadata.getInstance<TextInput>('classifyTitle').value,
        NRTIPOSOLICITACAO:Metadata.getInstance<Select>('classifyType').value, 
        IDPRIORIDADE: Metadata.getInstance<Select>('classifyPriority').value,  
        NRFUNCRESPONSAVEL: Metadata.getInstance<Select>('classifyResponsible').value,  
        NRCHECKLIST: Metadata.getInstance<Select>('classifyChecklist').value,  
        DSSOLICITACAO: Metadata.getInstance<Textarea>('classifyDescription').value,
        NRLOCALCLIENTE: this.requestParams.requestNrLocation,
        DTHISTSOLICITACAO: this.requestParams.requestOpenOn,
        NRFUNCSOLICITANTE: localStorage.getItem('UserLogado'),
      };
      // console.log(params);
      Http.post('/classifyRequest', { params }).then((response: IDictionary) => {
        if(response.data.dataset.info.message != 'Classificado com sucesso') {
          DialogService.show({
            name: 'classificationFailed',
            type: 'error',
            title: 'Erro!',
            text: 'Não foi possível classificar a solicitação.',
          });
        } else {
          AlertService.show ({
            name: 'classificationConcluded',
            text: 'Solicitação classificada com sucesso.',
            color: 'success',
            top: true,
            left: false,
          });
          (getRoutePath() === '/solicitacoesDeTarefas') ? this.getRequests() : this.getRequestInfo(); // Reload no component devido a tela aberta no momento
        }
      });

      this.modalSolicitacoes.closeClassifyModal();
    }
  }

}
